# This file is automatically generated by gendoc

from . import server, _server
@server.portable_class('anvil.Component')
class Component(_server.SerializeWithIdentity):
    def __init__(self,__ignore_property_exceptions=False,**props):
        self.__dict__.update(props)
    def __deserialize__(self, data, global_data):
        self.__dict__.update(data)
    def __serialize_once__(self, global_data):
        return self.__dict__

@server.portable_class('anvil.Container')
class Container(Component):
    def __init__(self,__ignore_property_exceptions=False,**props):
        self.__dict__['$_components'] = []
        self.__dict__.update(props)
    def add_component(self, c, **lp):
        self.__dict__['$_components'].append((c, lp))

@server.portable_class('anvil.ComponentTag')
class ComponentTag(object):
    pass

@server.portable_class('anvil.Button')
class Button(Component):
    pass

@server.portable_class('anvil.Canvas')
class Canvas(Component):
    pass

@server.portable_class('anvil.CheckBox')
class CheckBox(Component):
    pass

@server.portable_class('anvil.ColumnPanel')
class ColumnPanel(Container):
    pass

@server.portable_class('anvil.DataGrid')
class DataGrid(Container):
    pass

@server.portable_class('anvil.DataRowPanel')
class DataRowPanel(Container):
    pass

@server.portable_class('anvil.DatePicker')
class DatePicker(Component):
    pass

@server.portable_class('anvil.DropDown')
class DropDown(Component):
    pass

@server.portable_class('anvil.FileLoader')
class FileLoader(Component):
    pass

@server.portable_class('anvil.FlowPanel')
class FlowPanel(Container):
    pass

@server.portable_class('anvil.GoogleMap')
class GoogleMap(Container):
    pass

@server.portable_class('anvil.GridPanel')
class GridPanel(Container):
    pass

@server.portable_class('anvil.HtmlTemplate')
class HtmlTemplate(Container):
    pass

@server.portable_class('anvil.Image')
class Image(Component):
    pass

@server.portable_class('anvil.Label')
class Label(Component):
    pass

@server.portable_class('anvil.LinearPanel')
class LinearPanel(Container):
    pass

@server.portable_class('anvil.Link')
class Link(Container):
    pass

@server.portable_class('anvil.Plot')
class Plot(Component):
    pass

@server.portable_class('anvil.RadioButton')
class RadioButton(Component):
    pass

@server.portable_class('anvil.RepeatingPanel')
class RepeatingPanel(Component):
    pass

@server.portable_class('anvil.RichText')
class RichText(Container):
    pass

@server.portable_class('anvil.Spacer')
class Spacer(Component):
    pass

@server.portable_class('anvil.TextArea')
class TextArea(Component):
    pass

@server.portable_class('anvil.TextBox')
class TextBox(Component):
    pass

@server.portable_class('anvil.Timer')
class Timer(Component):
    pass

@server.portable_class('anvil.XYPanel')
class XYPanel(Container):
    pass

@server.portable_class('anvil.YouTubeVideo')
class YouTubeVideo(Component):
    pass

